//  ********************
//
//  SQLQueries.vw 
//
//  Created by R.Worsley  
//  rlworsley@gmail.com
//
//  ********************
//  Revision Log
//  08/05/2020 Created
//  09/03/2020 Revise going to tab 2
//  09/04/2020 Add support to SavedQueries for both embedded and SQL 
//
Use Windows.pkg
Use DFClient.pkg
Use cTextEdit.pkg

Use SQLScripts.pkg

Use Global.pkg
Use cSQDbForm.pkg

Use SavedQueries.dd
Use SysConfig.dd
Use SavedQueries.dg
Use DFEntry.pkg
Use dfTabDlg.pkg
Use cCJGrid.pkg
Use cCJGridColumn.pkg
Use SQLTableLookup.dg
Use SQLColumnLookup.dg
Use SQLQueriesNotes.dg
Use cSQDbForm.pkg

Open codemast

Register_Object oSQLQueriesCSVRpt
Register_Object oExport_btn

Deferred_View Activate_oSQLQueries for ;
Object oSQLQueries is a dbView
    
    Set Border_Style to Border_Thick
    Set Size to 297 425
    Set piMinSize to 227 425
    Set Location to 2 4
    Set Label to "SQL Queries"
    Set Auto_Clear_DEO_State to False
    Set Verify_Data_Loss_msg to (RefFunc(No_Confirmation))
    Set Verify_Save_msg to (RefFunc(No_Confirmation))

    Property String  psModuleName       "SQL Query Report"
    Property String[][] psaResults
    Property String[] psaNotAllowed
    Property String[] psaInjection
    Property String[] psaTables
    Property String[] psaColumns
    Property String[] psaColHeaders
    Property String   psUsingDocName          "UsingSQLQueries.chm"
    Property String   psUsingDocNameAndPath   ""

    Object oSysconfig_DD is a Sysconfig_DataDictionary
    End_Object

    Object oSavedQueries_DD is a SavedQueries_DataDictionary
        Procedure OnLoadRecord Integer iCurRec
            Set Enabled_State of oNotes_Btn to (iCurRec > 0)
        End_Procedure

        Procedure Clear
            String[][] saResults
            tDataSourceRow[] TheData
            Forward Send Clear
            Set Value of oCurrentTable to ""
            Set psaResults to saResults
            If (IsComObjectCreated(oResultsGrid)) Begin 
                Send InitializeData to oResultsGrid TheData
                Send DestroyColumnObjects to oResultsGrid
            End
        End_Procedure
    
        Procedure Clear_All
            String[][] saResults
            tDataSourceRow[] TheData
            Forward Send Clear_All
            Set Value of oCurrentTable to ""
            Set psaResults to saResults
            Set Value of oRecsReceived to ""
            If (IsComObjectCreated(oResultsGrid)) Begin 
                Send InitializeData to oResultsGrid TheData
                Send DestroyColumnObjects to oResultsGrid
            End
        End_Procedure

    End_Object

    Set Main_DD to oSavedQueries_DD
    Set Server to oSavedQueries_DD
    
    Procedure Request_Save
        String  sQuery sDesc
        String[][] saResults
        If (HasRecord(oSavedQueries_DD)) Begin 
            Forward Send Request_Save
            Procedure_Return
        End
        Get Field_Current_Value of oSavedQueries_DD Field SavedQueries.query to sQuery
        If (sQuery = "") Get Value of oQuery to sQuery
        If (sQuery = "") Procedure_Return
        Set psQuery of oSavedQueries_dg to sQuery
        Send Popup to oSavedQueries_dg
        Get psQueryDescription of oSavedQueries_dg to sDesc
        Move gsLoggedUserLogin to SavedQueries.Owner
        Move sDesc             to SavedQueries.Description
        Send Find to oSavedQueries_DD GE 1
    End_Procedure
    
    Procedure doExecute
        Send OnClick to oProcess_Btn
    End_Procedure
    On_Key Key_Ctrl+Key_E  Send doExecute

    Procedure Entering_Scope Returns Integer
        Integer iRetVal
        Boolean bViewUser
        Forward Get msg_Entering_Scope to iRetVal
        // Initialize all
        Set Enabled_State of oProcess_Btn to True
        Set Enabled_State of oSavedQueries_description to True
        Send doPopulateArrays "NA"
        Send doPopulateArrays "INJ"
        Send doPopulateArrays "TABLES"
        Send doPopulateArrays "COLUMNS"
        Set Label to ("SQLQueries - " + SysConfig.Database)
        Get pbViewUser of oApplication to bViewUser
        Procedure_Return iRetVal
    End_Procedure

    // Put in an array for quicker validation        
    Procedure doPopulateArrays String sType
        Integer i
        String[] saArray
        
        If (sType = "NA") Get psaNotAllowed to saArray
        If (sType = "INJ") Get psaInjection to saArray
        If (sType = "TABLES") Get psaTables to saArray
        If (sType = "COLUMNS") Get psaColumns to saArray
        
        Move (ResizeArray(saArray, 0)) to saArray
        Clear codemast
        Move sType to codemast.type
        Find GT codemast by 1
        While (Found and codemast.type = sType) 
            Move codemast.code to saArray[i]
            Increment i
            Find GT codemast by 1
        Loop
        If (sType = "NA") Set psaNotAllowed to saArray
        If (sType = "INJ") Set psaInjection to saArray
        If (sType = "TABLES") Set psaTables to saArray
        If (sType = "COLUMNS") Set psaColumns to saArray
    End_Procedure
    
    Object oQueryTabDialog is a dbTabDialog
        Set Size to 225 412
        Set Location to 29 7
    
        Set Rotate_Mode to RM_Rotate
        Set peAnchors to anAll

        Procedure Request_Switch_To_Tab Integer tab# Integer PointerMode
            Forward Send Request_Switch_To_Tab tab# PointerMode
            If (tab# = 0) Begin
                Set Enabled_State of oProcess_Btn to True
                Set Enabled_State of oExport_btn to False
                Set Enabled_State of oTable_Btn to True
                Set Enabled_State of oColumn_btn to True
                Set Enabled_State of oSavedQueries_description to True
            End
            If (tab# = 1) Begin
                Set Enabled_State of oProcess_Btn to False
                Set Enabled_State of oExport_btn to True
                Set Enabled_State of oTable_Btn to False
                Set Enabled_State of oColumn_btn to False
                Set Enabled_State of oSavedQueries_description to False
            End
        End_Procedure
        
        Object oQuery_tp is a dbTabPage
            Set Label to "Query"

            Object oQuery is a cdbTextEdit
                Entry_Item SavedQueries.query
                Set Size to 202 396
                Set Location to 5 5
                Set peAnchors to anAll
                Set pbWrap to False
                Set Verify_Data_Loss_msg to (RefFunc(No_Confirmation))
                Set Verify_Save_msg to (RefFunc(No_Confirmation))
            End_Object

        End_Object

        Object oResults_tp is a dbTabPage
            Set Label to "Results"

            Object oResultsGrid is a cCJGrid
                Set Size to 201 395
                Set Location to 6 7
                Set peAnchors to anAll
                Set pbAllowDeleteRow to False
                Set pbAllowInsertRow to False
                Set pbAutoAppend to False
                Set piCaptionBackColor to clMenu
                Set piCaptionForeColor to clGreen
                Set pbAutoColumnSizing to False
                Set pbAllowAppendRow to False
                Set pbEditOnClick to True
                Set pbReadOnly to True

                Procedure LoadData Integer iColCount 
                    tDataSourceRow[] TheData
                    Integer iRow iRows iCol i 
                    Integer[] iaObjects
                    String[][] saResults
                    
                    // Get the datasource indexes of the various columns
                    Get psaResults to saResults
                    Move (SizeOfArray(saResults)) to iRows
                    If (iColCount = 0) Move (SizeOfArray(saResults[1])) to iColCount
                    // Load all data into the datasource array
                    For iRow from 0 to (iRows - 1)
                        For iCol from 0 to (iColCount -1)
                            Move saResults[iRow][iCol] to TheData[iRow].sValue[iCol] 
                        Loop
                    Loop
                    // Initialize Grid with new data
                    Send InitializeData TheData
                    Send MovetoFirstRow
                End_Procedure
            
                Procedure doBuildGridColumns Integer iColCount String sTableName String sAsterisk String sSQL
                    Integer i iCol iPos iLen iArraysize
                    Handle[] haGridObj
                    Handle  hoCol
                    String  sColumns sStartStr sReturnStr sCRLF
                    String[][] saColumns
                    String[] saColHeaders
                    
                    Move (Replace(";", sTableName, "")) to sTableName
                    Move (Character(13) + Character(10)) to sCRLF
                    Get psaColHeaders to saColHeaders
                    
                    For i from 0 to (iColCount - 1)
                        Get Create (RefClass(cCJGridColumn)) to hoCol 
                        Move hoCol to haGridObj[i]
                        If (iColCount > 20) Set piWidth of hoCol to 60
                        Else Set piWidth of hoCol to 100
                        If (sTableName <> "") Begin // or sAsterisk <> "Y") Begin 
                            Set psCaption of hoCol to saColHeaders[i]
                        End
                    Loop

                    Set phoColumnObjects to haGridObj
                    Send CreateColumns
                    Send LoadData iColCount 
                End_Procedure
                
            End_Object
           
        End_Object

    End_Object

    Object oSavedQueries_description is a cSQdbForm
        Entry_Item SavedQueries.description
        Set Location to 5 71
        Set Size to 13 231
        Set Label to "Saved Queries:"
        Set Label_Col_Offset to 2
        Set Label_Justification_Mode to JMode_Right
        Set Prompt_Button_Mode to PB_PromptOn
        Set Entry_State to False
        
        Procedure Prompt_Callback Handle hoPrompt
            Send doSetUpList to hoPrompt
        End_Procedure
        
        Procedure Prompt
            Integer iID iIndex
            Boolean bRecnumTable
            Forward Send Prompt
            Get_Attribute DF_FILE_RECNUM_TABLE of SavedQueries.File_Number to bRecnumTable
            If (bRecnumTable) Begin 
                Get Field_Current_Value of (oSavedQueries_DD(oSqlsaved_sl)) Field SavedQueries.Recnum to iID
                Move 0 to iIndex
            End
            Else Begin
                // assumes that if this table is converted to SQL that it will have an ID
                // Get Field_Current_Value of oSavedQueries_DD Field SavedQueries.ID to iID
                // Move 1 to iIndex
            End
            If (iID > 0) Begin
                Send Clear to oSavedQueries_DD
                Move iID to SavedQueries.Recnum
                Send Find to oSavedQueries_DD EQ iIndex
            End
        End_Procedure
    End_Object

    Object oExit_Btn is a Button
        Set Location to 261 366
        Set Label to "Exit"
        Set peAnchors to anBottomRight
    
        Procedure OnClick
            If (not(HasRecord(oSavedQueries_DD))) Set Changed_State of oSavedQueries_DD to False // so no "changed" message when exit
            Send Exit_Application
        End_Procedure
    
    End_Object

    Object oProcess_Btn is a Button
        Set Location to 261 309
        Set Label to "Execute"
        Set peAnchors to anBottomRight
        
        Property String psSQL
        Property String psTableName
        
        Function iGetCountCommas String sSQL Returns Integer
            String  sStartStr sReturnStr sValue
            Integer iCount
            If (Pos(",", sSQL)= 0) Function_Return 0 
            Move sSQL to sStartStr
            Repeat 
                Send doParseString "," (&sStartStr) (&sReturnStr)
                Move sReturnStr to sValue
                If (sStartStr <> "") Increment iCount
            Until (sStartStr = "")  
            Function_Return iCount
        End_Function

        Function doTestTableName String sSQL Returns String
            Integer i iLen iPos
            String  sTableName sCRLF sSchema
            Move (Character(13) + Character(10)) to sCRLF 
            Move (Trim(SysConfig.Table_Schema) + ".") to sSchema
            Move 0 to i
            Repeat
                Move (Replaces(sCRLF, sSQL, " ")) to sSQL
                Move (Replaces ("[", sSQL, "")) to sSQL
                Move (Replaces ("]", sSQL, "")) to sSQL
                Move (Replace(sSchema, sSQL, "")) to sSQL
                Move (Pos("from", lowercase(sSQL))) to iPos
                Move (Length(sSQL)) to iLen
                Move (right(sSQL, (iLen - (iPos+3) ))) to sTableName
                Move (Replace( ";", trim(sTableName),"")) to sTableName
                Move (Trim(sTableName)) to sTableName
                // now test if anything following the table name
                Move (Pos(" ", sTableName)) to iPos
                If (iPos > 0) Move (trim(Left(sTableName, iPos))) to sTableName
                If (iPos = 0 and Left(sTableName, 1) = "v") Begin 
                    Set psTableName to (trim(sTableName))
                    Set Enabled_State of oExport_btn to True
                    Function_Return True // no where clause
                End
                Move sTableName to sSQL
                If (iPos > 0) Move (Left(sTableName, iPos)) to sTableName
                If (Left(sTableName, 1) <> "v") Begin
                    Set psTableName to (trim(sTableName)) 
                    Set Enabled_State of oExport_btn to True
                    Function_Return False
                End
                Increment i
            Until (not(lowercase(sSQL) contains "from"))
            Set psTableName to (trim(sTableName))
            Set Enabled_State of oExport_btn to True
            Function_Return True
        End_Function

        Function doTestJoinName String sSQL Returns Boolean
            Integer i iLen iPos
            String  sTableName 
            Move (Lowercase(sSQL)) to sSQL
            Move 0 to i
            If (not(sSQL contains "join ")) Function_Return True
            If (pbViewUser(oApplication) = False) Function_Return True
            // If view user, test all tables in the joins for leading "v"
            Repeat
                Move (Pos("join", sSQL)) to iPos
                Move (Length(sSQL)) to iLen
                Move (right(sSQL, (iLen - iPos +1))) to sTableName
                Move (Replace( "join ", trim(sTableName),"")) to sTableName
                Move (Pos(" ", sTableName)) to iPos
                Move sTableName to sSQL
                Move (Left(sTableName, iPos)) to sTableName
                If (Left(sTableName, 1) <> "v") Function_Return False
                Increment i
            Until (not(sSQL contains "join"))
            Function_Return True
        End_Function

        Procedure OnClick
            String  sSQL sTest sSelected sTableName sTable sAsterisk sCRLF sStartStr sReturnStr
            Integer i iCol iColCount iArraySize iObj iPos iCnt iPosCom
            Boolean bViewUser bOkJoin bOkFrom bAsterisk
            String[][] saResults
            String[] saNotAllowed saInjection saTables saColumns saQuery saColHeaders
            
            Get SelText of oQuery to sSelected
            Get pbViewUser of oApplication to bViewUser
            Move (Character(13)+Character(10)) to sCRLF
            Set Value of oRecsReceived to ""
            
            // Select id, short_description, priority, affected_module from tasktracker
            If (trim(sSelected) <> "") Begin 
                Move sSelected to sSQL
            End
            Else Begin
                Get Value of oQuery to sSQL
            End
            // basic check
            If (not(lowercase(sSQL) contains "select") and not(lowercase(sSQL) contains "from")) Begin 
                Send Stop_Box "This query isn't properly structured, please try again."
                Procedure_Return
            End

            If (Lowercase(sSQL) contains "join" and sSQL contains " * ") Begin
                Send Info_Box "Warning: Joins with '*' will not return correct column headers for the joined table. Specify the column names for correct headers."
            End
            
            // If an embedded semicolon separating two statements, lose the second one
            Move (Pos(";", sSQL)) to iPos
            If (iPos > 0) Move (Left(sSQL, iPos+1)) to sSQL
            
            Move (Replace("select", lowercase(sSQL), "")) to sTest
            If (sTest contains "select") Begin 
                Send Stop_Box "This query contains more than one query, you can only run one at a time!"
                Procedure_Return
            End
            
            // remove comments or they may show up in the column header text
            Move (Pos("--", sSQL)) to iPosCom 
            If (iPosCom > 0) Begin
                Move sSQL to sStartStr 
                Move 0 to iCnt
                Repeat
                    Send doParseString sCRLF (&sStartStr) (&sReturnStr)
                    Move (Replaces(sCRLF, sReturnStr, "")) to sReturnStr
                    Move (Pos("--", sReturnStr)) to iPosCom 
                    If (iPosCom) Begin 
                        Move (left(sReturnStr, (iPosCom - 1))) to sTest
                    End
                    Else Move sReturnStr to sTest
                    Move sTest to saQuery[iCnt]
                    Increment iCnt
                Until (sStartStr = "")
                
                Move (SizeOfArray(saQuery)) to iArraySize
                Move "" to sSQL
                For i from 0 to (iArraySize - 1)
                    Append sSQL (" " + saQuery[i])
                Loop    
            End

            // 3 conditions for determining column names
            // select * with 
            // select distinct field names 
            // query with a join -- no determine columns
            Move (lowercase(sSQL) contains " * ") to bAsterisk
            
            If (bAsterisk) Begin 
                Move "Y" to sAsterisk
                // select * from wwsys
                Get doTestTableName sSQL to bOkFrom
                // do we have a correct table name?
                If (TableValidate(Self, psTablename(Self))= False) Begin
                    Send Stop_Box "The main table name isn't correct, the query won't run. Please correct it."
                    Procedure_Return
                End
                
                Get doTestJoinName sSQL to bOkJoin
                If (bViewUser and (not(bOkFrom) or not(bOkJoin))) Begin 
                    If (not(bOkFrom)) Send Stop_Box "You are limited to running 'View' queries only."
                    Else Send Stop_Box "You are limited to running 'View' joins only."
                    Procedure_Return
                End
                Get psTableName to sTableName
                If (lowercase(sTableName) contains "join") Move "" to sTableName  // Other bits in this, no can pull column names
                Else Begin 
                    // remove anything after the table name if a where clause exists
                    Move (Pos(" ", sTableName)) to iPos
                    If (iPos > 0) Begin 
                        Move (Left(sTableName, iPos)) to sTableName
                        Move (Trim(sTableName)) to sTableName
                        If (bViewUser and (not(bOkFrom) or not(bOkJoin))) Begin 
                            If (not(bOkFrom)) Send Stop_Box "You are limited to running 'View' queries only."
                            Else Send Stop_Box "You are limited to running 'View' joins only."
                            Procedure_Return
                        End
                    End
                End
            End
            Else Begin 
                Get doTestTableName sSQL to bOkFrom
                Get doTestJoinName sSQL to bOkJoin
                If (bViewUser and (not(bOkFrom) or not(bOkJoin))) Begin 
                    If (not(bOkFrom)) Send Stop_Box "You are limited to running 'View' queries only."
                    Else Send Stop_Box "You are limited to running 'View' joins only."
                    Procedure_Return
                End
            End

            Move (Trim(sSQL)) to sSQL
            If (Right(sSQL, 1) <> ";") Move (sSQL + ";") to sSQL
            
            If (Trim(sSQL) = "") Begin 
                Send Stop_Box "It would be nice to have a query of some sort!"
                Procedure_Return
            End

            If (sSQL = "") Procedure_Return
            
            Set Enabled_State of oSavedQueries_description to False
            Get psaNotAllowed to saNotAllowed
            Get psaInjection to saInjection
            Get psaTables to saTables
            Get psaColumns to saColumns
            
            Move (SizeOfArray(saNotAllowed)) to iArraySize
            Decrement iArraySize
            For i from 0 to iArraySize
                If (uppercase(sSQL) contains uppercase(trim(saNotAllowed[i]))) Begin
                    Send Stop_Box ("The SQL statement contains reserved word '" + trim(saNotAllowed[i]) + "' which is not allowed, see the help.") "Error!"
                    Procedure_Return
                End
            Loop

            Move (SizeOfArray(saInjection)) to iArraySize
            Decrement iArraySize
            For i from 0 to iArraySize
                If (uppercase(sSQL) contains uppercase(trim(saInjection[i]))) Begin
                    Send Stop_Box ("The SQL statement contains the phrase '" + trim(saInjection[i]) + "' which is not allowed, see the help.") "Error!"
                    Procedure_Return
                End
            Loop
            
            Move (SizeOfArray(saTables)) to iArraySize
            Decrement iArraySize
            For i from 0 to iArraySize
                If (sSQL contains trim(saTables[i])) Begin
                    Send Stop_Box ("Table '" + trim(saTables[i]) + "' is not permitted, see the help.") "Error!"
                    Procedure_Return
                End
            Loop
            
            Move (SizeOfArray(saColumns)) to iArraySize
            Decrement iArraySize
            For i from 0 to iArraySize
                If (sSQL contains trim(saColumns[i])) Begin
                    Send Stop_Box ("Column '" + trim(saColumns[i]) + "' is not permitted, see the help.") "Error!"
                    Procedure_Return
                End
            Loop

            Send DestroyColumnObjects to oResultsGrid

            // Remove the ability to concatinate statements
            Move (Pos(";", sSQL)) to iPos
            Move (Left(sSQL, iPos)) to sSQL

            Set psSQL to sSQL

            // don't do this for update, delete, etc
            If (lowercase(sSQL) contains "delete" or Lowercase(sSQL) contains "update " or ;
                    Lowercase(sSQL) contains "insert " or Lowercase(sSQL) contains "alter " or ;
                    Lowercase(sSQL) contains "drop " or Lowercase(sSQL) contains "truncate" or ;
                    Lowercase(sSQL) contains "create ") Begin
                Send Stop_Box "Insert, Update Delete and other such commands not allowed!"
                Procedure_Return
            End

            Send Cursor_wait to Cursor_Control
            //---------------------------------------------->
            // Main Query
            //---------------------------------------------->
            Get SQLSelect sSQL True to saResults
            Send Cursor_Ready to Cursor_Control
            
            Move (SizeOfArray(saResults)) to iArraySize
            Set Value of oRecsReceived to (String(iArraySize-1) + " Records")
            If (iArraySize = 0) Begin 
                Send Info_Box "No results found!"
                //SQL_ERROR_MESSAGE sErrorText
                Procedure_Return
            End
            If (lowercase(sSQL) contains "select") Begin
                Set psaResults to saResults
                Move (SizeOfArray(saResults[0])) to iColCount  // Need for system records
            End
                        
            // parse out the column headers which is row 0
            Move (SizeOfArray(saResults[0])) to iColCount
            For i from 0 to (iColCount - 1)
                Move saResults[0][i] to saColHeaders[i]
            Loop
            // remove the column headers from the data - row 0
            Move (RemoveFromArray(saResults, 0)) to saResults
            
            Set psaResults to saResults
            Set psaColHeaders to saColHeaders
                        
            // jump to oResults tab
            Send Request_Switch_To_Tab to oQueryTabDialog 1 3
            
            Get psTableName to sTableName
            Send doBuildGridColumns to oResultsGrid iColCount sTableName sAsterisk sSQL
        End_Procedure
    
    End_Object

    Object oTextBox1 is a TextBox
        Set Auto_Size_State to False
        Set Size to 10 184
        Set Location to 19 99
        Set Label to "A query does not need to be saved in order to be run."
        Set FontWeight to fw_Bold
    End_Object

    Object oExport_btn is a Button
        Set Size to 14 55
        Set Location to 261 246
        Set Label to "Export"
        Set peAnchors to anBottomRight
        Set focus_mode to Pointer_Only 
    
        Procedure OnClick
            Integer iArraySize 
            String  sTitle
            String[][] saResults
            String[] saColHeaders
            
            Get psaResults to saResults
            Move (SizeOfArray(saResults)) to iArraySize
            If (iArraySize = 0) Begin
                Send Stop_Box "Run a query first!"
                Procedure_Return
            End
            Get psaColHeaders to saColHeaders
            Set psaResults    of oSQLQueriesCSVRpt to saResults
            Set psaColHeaders of oSQLQueriesCSVRpt to saColHeaders
            Get Value of oSavedQueries_description to sTitle
            Set psTheTitle of oSQLQueriesCSVRpt to sTitle 
            Send Activate_oSQLQueriesCSVRpt
        End_Procedure
    
    End_Object

    Object oTable_Btn is a Button
        Set Size to 14 53
        Set Location to 261 63
        Set Label to "Tables && Views"
        Set peAnchors to anBottomLeft
        Set focus_mode to Pointer_Only 
    
        Procedure OnClick
            String  sLogical sQuery sJustLook
            Integer iPosfrom iPosSelect
            Boolean bCancel
            Send Popup to oSQLTableLookup_sl
            Get psTableName of oSQLTableLookup_sl to sLogical
            Get pbCancel of oSQLTableLookup_sl to bCancel
            Get psJustLookTable of oSQLTableLookup_sl to sJustLook
            If (bCancel) Procedure_Return
            Set Value of oCurrentTable to sLogical
            If (sLogical = "") Begin 
                Set Value of oCurrentTable to sJustLook
                Procedure_Return
            End
            Get Value of oQuery to sQuery
            Move (Pos("SELECT", Uppercase(sQuery))) to iPosSelect
            Move (Pos("FROM", Uppercase(sQuery))) to iPosfrom
            Move (If(iPosSelect = 0, "Select", "") * sQuery * (If(iPosfrom = 0, "from", "")) * sLogical) to sQuery
            Send Delete_Data to oQuery
            Set Value of oQuery to (sQuery + " ")
            Move (Pos("FROM", Uppercase(sQuery))) to iPosfrom
            // position the cursor just before the word 'from'
            Send SetSel of oQuery (iPosfrom - 1) (iPosfrom - 1)
        End_Procedure
    
    End_Object

    Object oColumn_btn is a Button
        Set Location to 261 180
        Set Label to "Column List"
        Set peAnchors to anBottomLeft
        Set focus_mode to Pointer_Only 
        
        Procedure OnClick
            String  sLogical sQuery  sColumn sLeft sRight
            String[] aColumn
            Integer iCursorPos iLenField
            Get Value of oCurrentTable to sLogical
            If (sLogical = "") Begin 
                Send Stop_Box "Select a table first!"
                Procedure_Return
            End
            Set psTableName of oSQLColumnLookup_sl to sLogical
            Send Popup to oSQLColumnLookup_sl
            Get paColumnName of oSQLColumnLookup_sl to aColumn
            Move (StrJoinFromArray(aColumn, ", ")) to sColumn
            Get SelStart of oQuery to iCursorPos
            Get Value of oQuery to sQuery
            Move (Length(sColumn) + 2) to iLenField 
            Move (Left(sQuery, iCursorPos)) to sLeft
            Move (Right(trim(sQuery), Length(Trim(sQuery)) - iCursorPos)) to sRight  
            If (trim(sRight) = trim(sQuery)) Move "" to sRight  // no "from tablename"
            If (sColumn <> "") Begin 
                Move (trim(sLeft) * trim(sColumn) + ", " * trim(sRight)) to sQuery
                If (trim(sQuery) > ",") Begin 
                    Send Delete_Data to oQuery
                    Set Value of oQuery to sQuery
                    Send SetSel of oQuery (iCursorPos + iLenField) (iCursorPos + iLenField)
                End
            End
        End_Procedure
    
    End_Object

    Object oCurrentTable is a TextBox
        Set Size to 10 45
        Set Location to 263 119
        Set peAnchors to anBottomLeft
        Set focus_mode to Pointer_Only 
        
        Procedure Mouse_Click Integer iWindowNumber Integer iPosition
            String sQuery sLogical
            Forward Send Mouse_Click iWindowNumber iPosition
            Get Value of oQuery to sQuery 
            Get Value to sLogical
            Move (trim(sQuery) * trim(sLogical)) to sQuery
            Set Value of oQuery to sQuery
        End_Procedure
        
    End_Object

    Object oNotes_Btn is a Button
        Set Location to 261 9
        Set Label to "Notes"
        Set peAnchors to anBottomLeft
        Set focus_mode to Pointer_Only 
    
        Procedure OnClick
            Integer iID
            Boolean bRecnumTable
            Get_Attribute DF_FILE_RECNUM_TABLE of SavedQueries.File_Number to bRecnumTable
            If (bRecnumTable) Begin 
                Get Field_Current_Value of oSavedQueries_DD Field SavedQueries.Recnum to iID
            End
            Else Begin
                // assumes that if this table is converted to SQL that it will have an ID
                // Get Field_Current_Value of oSavedQueries_DD Field SavedQueries.ID to iID
            End
            Set piId of oSQLQueriesNotes to iID
            Send Popup to oSQLQueriesNotes
        End_Procedure
    
    End_Object

    Object oBN_Help is a Button
        Set Location to 279 9
        Set Label to "Help"
        Set peAnchors to anBottomLeft
        Set focus_mode to Pointer_Only 
    
          Procedure OnClick
            String sHelpPath
            Handle hoWorkspace
            Get phoWorkspace of ghoApplication to hoWorkspace
            Get psHelpPath of hoWorkSpace to sHelpPath
            Get vFolderFormat sHelpPath to sHelpPath
            Send vShellExecute "Open" (psUsingDocName(Self)) "" sHelpPath
          End_Procedure 
      
    End_Object

    Object oRecsReceived is a TextBox
        Set Auto_Size_State to False
        Set Size to 10 107
        Set Location to 282 310
        Set Label to ""
        Set peAnchors to anBottomRight
    End_Object
    
Cd_End_Object

